-- Custom hud for Specki, and Maril. Using the Custom Hud Functions library to prevent compatibility issues with other mods that use it.
-- Specki's hud will be SMW-themed. Maril's is still unknown... Sonic Advance.

local modname = "speckichars"
local vanilla = "vanilla"

local function isSpeckiOrMaril(skin)
	if skin == #skins["specki"] then return "specki" end
	if skin == #skins["maril"] then return "maril" end
	return neither
end

local function speckiScore(v, p)
	v.drawString(320-16, 32, p.score, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT, "right")
end

local function marilScore(v, p)
	v.drawNum(92, 12, p.score, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
end

local function speckiTime(v, p)
	local time = tostring(leveltime/TICRATE)

	if string.len(time) < 3 then
		repeat
			time = "0" .. $
		until string.len(time) >= 3
	end

	v.drawString(184, 24, "TIME", V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
	v.drawString(200, 32, time, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT, "center")
end

local function marilTime(v, p)
	local colon = v.cachePatch("STTCOLON")
	
	v.drawNum(44, 24, G_TicsToMinutes(leveltime), V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.draw(44, 24, colon, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.drawPaddedNum(68, 24, G_TicsToSeconds(leveltime), 2, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.draw(68, 24, colon, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.drawPaddedNum(92, 24, G_TicsToCentiseconds(leveltime), 2, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
end

local function speckiRings(v, p)
	local ring = v.cachePatch("RINGS1")
	local x = v.cachePatch("RINGS2")
	
	v.drawScaled(248*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, ring, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
	v.drawScaled(256*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, x, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
	v.drawString(304, 24, p.rings, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT, "right")
end

local function marilRings(v, p)
	local ring = v.getSpritePatch(SPR_RING, 7, 0)
	local rings = tostring(p.rings)

	v.drawScaled(19*FRACUNIT, 31*FRACUNIT, FRACUNIT/3*2, ring, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)

	if string.len(rings) > 3 then
		v.drawNum(36, 16, p.rings, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	elseif string.len(rings) > 2 then
		v.drawNum(32, 16, p.rings, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	elseif string.len(rings) > 1 then
		v.drawNum(28, 16, p.rings, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	else
		v.drawNum(24, 16, p.rings, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	end
end

local function speckiLives(v, p)
	local name = v.cachePatch("SPECKI")
	local x = v.cachePatch("RINGS2")

	v.drawScaled(16*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, name, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap("specki", p.skincolor))
	v.drawScaled(24*FRACUNIT, 32*FRACUNIT, FRACUNIT/3, x, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.drawString(40, 32, p.lives, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
end

local function marilLives(v, p)
	if p.lives ~= INFLIVES then
		local icon = v.getSprite2Patch("maril", SPR2_LIFE, false, 0, 0)
		local lives = tostring(p.lives)
	
		v.draw(18, 200-8, icon, V_PERPLAYER|V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT, v.getColormap("maril", p.skincolor))
	
		if string.len(lives) > 1 then
			v.drawNum(46, 200-21, p.lives, V_PERPLAYER|V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
		else
			v.drawNum(38, 200-21, p.lives, V_PERPLAYER|V_HUDTRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT)
		end
	end
end

local function specki(v, p)
	local emerald = v.cachePatch("EMMY")
	local x = v.cachePatch("RINGS2")
	local itembox = v.cachePatch("ITEMBOX")
	local speckiface = v.getSprite2Patch("specki", SPR2_XTRA, false, 0, 0)
	local emmynum = 0
	
	if emeralds & EMERALD1 then
		v.drawScaled(68*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_EMERALD))
		emmynum = $+1
	end
	
	if emeralds & EMERALD2 then
		v.drawScaled(76*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_PURPLE))
		emmynum = $+1
	end
	
	if emeralds & EMERALD3 then
		v.drawScaled(84*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_BLUE))
		emmynum = $+1
	end
	
	if emeralds & EMERALD4 then
		v.drawScaled(92*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_CYAN))
		emmynum = $+1
	end
	
	if emeralds & EMERALD5 then
		v.drawScaled(100*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_ORANGE))
		emmynum = $+1
	end
	
	if emeralds & EMERALD6 then
		v.drawScaled(108*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_RED))
		emmynum = $+1
	end
	
	if emeralds & EMERALD7 then
		v.drawScaled(116*FRACUNIT, 24*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_GREY))
		emmynum = $+1
	end
	
	v.drawScaled(100*FRACUNIT, 32*FRACUNIT, FRACUNIT/3, emerald, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT, v.getColormap(nil, SKINCOLOR_BLACK))
	v.drawScaled(108*FRACUNIT, 32*FRACUNIT, FRACUNIT/3, x, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	v.drawNum(132, 28, emmynum, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTOLEFT)
	
	if p.mo and p.mo.valid then
		v.drawScaled(152*FRACUNIT, 24*FRACUNIT, FRACUNIT/2, speckiface, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP, v.getColormap("specki", p.mo.color))
	else
		v.drawScaled(152*FRACUNIT, 24*FRACUNIT, FRACUNIT/2, speckiface, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP, v.getColormap("specki", p.skincolor))
	end
	
	v.drawScaled(144*FRACUNIT, 16*FRACUNIT, FRACUNIT/2, itembox, V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP)
end

local function maril(v, p)
	if not (p.mo and p.mo.valid) then return end
	if not p.mo.health then return end

	if p.maril_moveset == 1 then
		if not (p.mo.eflags & MFE_UNDERWATER) then
			if not (p.maril_spin or (p.pflags & PF_THOKKED) or P_PlayerInPain(p) or p.maril_fly or p.specki_sprung or p.maril_piano) then
				v.draw(hudinfo[HUD_LIVES].x, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARSPIN1"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
			else
				v.draw(hudinfo[HUD_LIVES].x, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARSPIN2"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
			end
		elseif not (p.maril_swimspincooldown or P_PlayerInPain(p) or p.specki_sprung or p.powers[pw_carry] or p.specki_sliding) then
			v.draw(hudinfo[HUD_LIVES].x, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARSPIN1"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
		else
			v.draw(hudinfo[HUD_LIVES].x, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARSPIN2"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
		end
		
		if p.maril_fly then
			v.draw(hudinfo[HUD_LIVES].x+34, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARFLY1"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
		elseif p.mo.state == S_PLAY_ROLL and (p.pflags & PF_THOKKED) and not p.maril_spin then
			v.draw(hudinfo[HUD_LIVES].x+34, hudinfo[HUD_LIVES].y - 32, v.cachePatch("MARSJMP1"), V_PERPLAYER|V_HUDTRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM, v.getColormap("maril", p.mo.color))
		end
	end
end

local transtimer = 0 -- get it? trans? :>

local function menu(v, p)
	if not (p.mo and p.mo.valid) then return end
	if not p.mo.health or (p.mo.skin ~= "specki" and p.mo.skin ~= "maril") then return end
	if p.speckichars_menu == nil or p.alivetime == nil or p.openedspeckichars_menu == nil then return end
	
	if not p.speckichars_menu and p.alivetime < 6*TICRATE and not p.openedspeckichars_menu then
		-- Let the player know they can access the menu if their actual alivetime is less than 6*TICRATE
		v.drawString(160, 88, "Want to change how", V_ALLOWLOWERCASE|V_SNAPTOTOP|V_HUDTRANSHALF|V_AQUAMAP, "center")
		v.drawString(160, 96, "you play as speckichars?", V_ALLOWLOWERCASE|V_SNAPTOTOP|V_HUDTRANSHALF|V_AQUAMAP, "center")
		v.drawString(160, 104, "Type 'specki_customize' in the console!", V_ALLOWLOWERCASE|V_SNAPTOTOP|V_HUDTRANSHALF|V_YELLOWMAP, "center")
		v.drawString(160, 112, "(or hold C2 for 3 seconds)", V_ALLOWLOWERCASE|V_SNAPTOTOP|V_HUDTRANSHALF|V_YELLOWMAP, "center")
	elseif p.speckichars_menu then
		-- The menu uses the player's skincolor as the primary color.
	
		if transtimer < 10 then
			v.fadeScreen(skincolors[p.skincolor].ramp[15], transtimer/2)
			transtimer = $ + 1
		else
			v.fadeScreen(skincolors[p.skincolor].ramp[15], 6)
		end
				
		-- Lets draw a preview of our character on the... left. Because why not?
		
		if p.mo.skin == "specki" then
			v.draw(64, 148, v.getSprite2Patch(p.mo.skin, SPR2_WALK, false, A+(leveltime/4%8), 2, 0), V_SNAPTOLEFT|V_FLIP, v.getColormap(p.mo.skin, p.skincolor))
		else
			-- maril has no walk cycle yet
			--v.draw(64, 152, v.getSprite2Patch(p.mo.skin, SPR2_WALK, false, A+(leveltime/4%8), 2, 0), V_SNAPTOLEFT|V_FLIP, v.getColormap(p.mo.skin, p.skincolor))
			v.draw(64, 152, v.getSprite2Patch(p.mo.skin, SPR2_STND, false, A, 2, 0), V_SNAPTOLEFT|V_FLIP, v.getColormap(p.mo.skin, p.skincolor))
		end
		
		v.drawString(68, 156, skins[p.mo.skin].realname, V_ALLOWLOWERCASE|V_SNAPTOLEFT|skincolors[p.skincolor].chatcolor, "center")
		
		-- Lets add a background to part of the right, so our text isn't just on transparency (might change in the future)
		
		v.drawFill(144, 0, 1920, 1080, skincolors[p.skincolor].ramp[12]|V_SNAPTORIGHT|V_SNAPTOTOP)
		
		-- placeholder text
		
		v.drawString(148, 8, "The menu isn't finished yet,\nsorry!", V_SNAPTORIGHT|V_SNAPTOTOP)
		
		-- speckichars version
		
		v.drawString(321, 196, "speckichars "..SPECKICHARSVERSION, V_ALLOWLOWERCASE|V_SNAPTORIGHT|V_SNAPTOBOTTOM|skincolors[p.skincolor].chatcolor, "small-right")
	elseif transtimer > 0 then
		v.fadeScreen(skincolors[p.skincolor].ramp[15], transtimer/2)
		transtimer = $ - 1
	end
end

local rankcolors = {
	["R"] = skincolors[SKINCOLOR_BLUE].chatcolor,
	["P"] = V_PURPLEMAP,
	["S"] = V_YELLOWMAP,
	["A"] = V_REDMAP,
	["B"] = V_SKYMAP,
	["C"] = V_PERIDOTMAP,
	["D"] = V_GRAYMAP
}

local rankbarcolors = {
	["R"] = skincolors[SKINCOLOR_BLUE].ramp[4],
	["P"] = skincolors[SKINCOLOR_PURPLE].ramp[4],
	["S"] = skincolors[SKINCOLOR_YELLOW].ramp[4],
	["A"] = skincolors[SKINCOLOR_RED].ramp[4],
	["B"] = skincolors[SKINCOLOR_SKY].ramp[4],
	["C"] = skincolors[SKINCOLOR_PERIDOT].ramp[4],
	["D"] = skincolors[SKINCOLOR_CARBON].ramp[4]
}

local function speckiPizza(v, p)
	if p.pizzaface then return end
	if PTSR == nil then return end
	if PTSR.maxrankpoints < 1 then return end
	if p.ptsr_rank == nil then return end
	
	if PTSR.intermission_tics and (PTSR.intermission_tics < 324)
	or (PTSR:inVoteScreen())
		return
	end
	
	local per = (PTSR.maxrankpoints)/8
	local rank = p.ptsr_rank
	local score = p.score
	local bar
	
	if rank == "P" then
		per = PTSR.maxrankpoints + PTSR.maxrankpoints/2
		score = $ - per
	elseif rank == "S" then
		per = PTSR.maxrankpoints
		score = $ - per
	elseif rank == "A" then
		per = $*4
		score = $ - per
	elseif rank == "B" then
		per = $*2
		score = $ - per
	elseif rank == "C" then
		score = $ - per
	end
	
	bar = FixedDiv(score, per)*80/FRACUNIT
	
	if score < PTSR.maxrankpoints + PTSR.maxrankpoints/2 then
		v.drawFill(320-112, 16, 80, 8, 31|V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
		v.drawFill(320-112, 16, bar, 8, rankbarcolors[rank]|V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
	else
		rank = "R"
	end
	
	v.drawString(320-24, 16, rank, rankcolors[rank]|V_PERPLAYER|V_HUDTRANS|V_SNAPTOTOP|V_SNAPTORIGHT)
end

hud.add(function(v, p)
	if speckichars.isSpeckiChar(p.skin) then
		customhud.SetupItem("score", modname)
		customhud.SetupItem("time", modname)
		customhud.SetupItem("rings", modname)
		customhud.SetupItem("lives", modname)
		customhud.SetupItem("specki", modname)
		customhud.SetupItem("maril", modname)
		menu(v, p)
	
		if isSpeckiOrMaril(p.skin) == "specki" then
			customhud.SetupItem("PTSR_rank", modname)
			speckiScore(v, p)
			speckiTime(v, p)
			speckiRings(v, p)
			speckiLives(v, p)
			specki(v, p)
			speckiPizza(v, p)
			
			if p.specki_sleeptimer == nil then return end
	
			if p.specki_sleeptimer >= 8*TICRATE then
				v.drawString(160, 56, "The specki has a nap.", nil, "center")
				v.drawString(160, 72, "Hold out! Specki!", nil, "center")
			end
		else
			customhud.SetupItem("PTSR_rank", "spicerunners")
			marilScore(v, p)
			marilTime(v, p)
			marilRings(v, p)
			marilLives(v, p)
			maril(v, p)
		end
	else
		customhud.SetupItem("score", vanilla)
		customhud.SetupItem("time", vanilla)
		customhud.SetupItem("rings", vanilla)
		customhud.SetupItem("lives", vanilla)
		customhud.SetupItem("PTSR_rank", "spicerunners")
	end
end)